//------------------------------
// Main.cpp
// Plasma Fractal testing
// Author: Kevin Fauteux
//------------------------------
#include "glut.h"
#include <windows.h>
#pragma comment( lib, "glut32.lib" )

#define MAX_X 512
#define MAX_Y 512
//--------------------------------
// Our Height Map
float HeightMap[MAX_X][MAX_Y];
int RedOn, GreenOn, BlueOn;
struct point{
	point(){ x = y = 0;}
	point( int nx, int ny ){ x = nx; y = ny;}
	int x, y;
};

//------------------
// GenerateNextLevel
void GenerateNextLevel( int x1, int y1, int x2, int y2 )
{
	float halfway = (float)x2 - float(x1);
	// we have gone as far as we can, our positions should be right beside each other
	if( halfway < 2.0f )
		return;

	point next5[5]; // next five points
	// top
	next5[0].x = x2 - ((x2 - x1)/2);
	next5[0].y = y1;
	HeightMap[next5[0].x][next5[0].y] = (HeightMap[x2][y1] + HeightMap[x1][y1])/2;
	// bottom
	next5[1].x = x2 - ((x2 - x1)/2);
	next5[1].y = y2;
	HeightMap[next5[1].x][next5[1].y] = (HeightMap[x2][y2] + HeightMap[x1][y2])/2;
	// left
	next5[2].x = x1;
	next5[2].y = y2 - ((y2 - y1)/2);
	HeightMap[next5[2].x][next5[2].y] = (HeightMap[x1][y2] + HeightMap[x1][y1])/2;
	// right
	next5[3].x = x2;
	next5[3].y = y2 - ((y2 - y1)/2);
	HeightMap[next5[3].x][next5[3].y] = (HeightMap[x2][y2] + HeightMap[x2][y1])/2;
	// center
	next5[4].x = x2 - ((x2 - x1)/2);
	next5[4].y = y2 - ((y2 - y1)/2);
	float average = HeightMap[x1][y1] + HeightMap[x2][y1] + HeightMap[x1][y2] + HeightMap[x2][y2];//HeightMap[x1][y1] + HeightMap[x2][y1] + HeightMap[x2][y1] + HeightMap[x2][y2];//
	average /= 4.0f;
	average += (((float)(rand()%10000)/10000.0f)-0.5f)*(float)(((float)x2-(float)x1)/(float)MAX_X);
	if( average > 1.0f )
		average = 1.0f;
	if( average < 0.0f )
		average = 0.0f;
	HeightMap[next5[4].x][next5[4].y] = average;

	// Call recursively
	GenerateNextLevel( x1, y1, next5[4].x, next5[4].y );
	GenerateNextLevel( next5[0].x, next5[0].y, next5[3].x, next5[3].y );
	GenerateNextLevel( next5[2].x, next5[2].y, next5[1].x, next5[1].y );
	GenerateNextLevel( next5[4].x, next5[4].y, x2, y2 );
}
void Clear( void )
{
	//-----------------------
	// Clear our height map
	HeightMap[0][0] += (float)(rand()%10000)/10000.0f-0.5f;
	HeightMap[0][MAX_Y] += (float)(rand()%10000)/10000.0f-0.5f;
	HeightMap[MAX_X][0] += (float)(rand()%10000)/10000.0f-0.5f;
	HeightMap[MAX_X][MAX_Y] += (float)(rand()%10000)/10000.0f-0.5f;
	GenerateNextLevel(0, 0, MAX_X, MAX_Y);
}

//------------------
// SetupRC
void SetupRC(void)
{
	//-----------------------
	// Clear our height map
	memset(HeightMap, 0, MAX_X*MAX_Y );
	srand(GetTickCount());
	//srand(1384);
	HeightMap[0][0] = (float)(rand()%10000)/10000.0f;
	HeightMap[0][MAX_Y] = (float)(rand()%10000)/10000.0f;
	HeightMap[MAX_X][0] = (float)(rand()%10000)/10000.0f;
	HeightMap[MAX_X][MAX_Y] = (float)(rand()%10000)/10000.0f;
	GenerateNextLevel(0, 0, MAX_X, MAX_Y);
	RedOn = GreenOn = BlueOn = 1;
}

//------------------
// ChangeSize
void ChangeSize(int w, int h)
{
	gluOrtho2D(0,w, 0,h);
}

//------------------
// RenderScene
void RenderScene(void)
{
	// Clear color and depth buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Set drawing color
	glColor3f(0.0f, 0.0f, 0.0f);

	// Draw something
	glRectf(-1.0f, 1.0f, 1.0f, -1.0f);  // Draw a 2D Rectangle

	// Set drawing color
	glBegin( GL_POINTS );
	if( RedOn | GreenOn | BlueOn )
	for( int i = 0; i < MAX_X; i++ )
	{
		for( int j = 0; j < MAX_Y; j++ )
		{
			float r, g, b;
			r = g = b = 0.0f;
			if( HeightMap[i][j] < 0.5f )
			{
				r = HeightMap[i][j] * 2.0f;
			}
			else
			{
				r = (1.0f - HeightMap[i][j]) * 2.0f;
			}

			if( HeightMap[i][j] >= 0.3f && HeightMap[i][j] < 0.8f )
			{
				g = (HeightMap[i][j] - 0.3f) * 2.0f;
			}
			else if( HeightMap[i][j] < 0.3f )
			{
				g = (0.3f - HeightMap[i][j]) * 2.0f;
			}
			else
			{
				g = (1.3f - HeightMap[i][j]) * 2.0f;
			}

			if( HeightMap[i][j] >= 0.5f )
			{
				b = (HeightMap[i][j] - 0.5f ) * 2.0f;
			}
			else
			{
				b = (0.5f - HeightMap[i][j]) * 2.0f;
			}
			r = ( RedOn ? r : 0.0f );
			g = ( GreenOn ? g : 0.0f );
			b = ( BlueOn ? b : 0.0f );
			glColor3f(r, g, b);
			glVertex2f( 10.0f+(GLfloat)i, 10.0f+(GLfloat)j );
		}
	}
	glEnd();

	glutSwapBuffers();	// Do buffer swap
	Sleep(200);
	glutPostRedisplay();	// Redraw (use for animation)
	//Clear();

	if( GetAsyncKeyState( 'R' ) )
	{
		RedOn = !RedOn;
	}
	if( GetAsyncKeyState( 'G' ) )
	{
		GreenOn = !GreenOn;
	}
	if( GetAsyncKeyState( 'B' ) )
	{
		BlueOn = !BlueOn;
	}
}

//------------------
// Main
int main( int argc, char* argv[] )
{
	glutInit( &argc, argv );
	glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
	glutInitWindowSize( MAX_X + 20, MAX_Y + 20);
	glutCreateWindow( "Plasma Fractals" );
	glutReshapeFunc( ChangeSize );
	glutDisplayFunc( RenderScene );

	SetupRC();

	glutMainLoop();

	return 0;
}